//---------------------------------------------------------------------------
// SelfImage - Disk image manipulation program
// SelfImage_TImageStore.h - TImageStore class header
// Copyright 2005, Kurt Fitzner <kfitzner@excelcia.org>
//---------------------------------------------------------------------------
// This file is part of SelfImage.
//
// SelfImage is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License (Version 2) as
// published by the Free Software Foundation.
//
// SelfImage is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//---------------------------------------------------------------------------
#ifndef SelfImage_TImageStoreH
#define SelfImage_TImageStoreH
//---------------------------------------------------------------------------
#include "SelfImage_TNBDClient.h"
#include "SelfImage_Exceptions.h"
#include "SelfImage_TypesDefs.h"
//---------------------------------------------------------------------------
struct TTargetGeometry {
  __int64 Bytes;
  __int64 Sectors;
  int BytesPerSector;
  int SectorsPerCluster;
};
class TImageStore {
  public:
    __property TImageStoreType    Type  = {read = FType};
    __property AnsiString         FileName = {read = FFileName};
    __property TTargetGeometry    Geometry = {read = FGeometry};
    __property bool               Compressed = {read = GetCompressed};
    __property bool               BlockCompressed = {read = GetBlockCompressed};
    __property TCompressionFormat CompressionFormat = {read = FCompressionFormat};
    __property unsigned __int64   Position = {read = FPosition};

    __fastcall TImageStore(TImageStoreType isType, AnsiString Target, bool Writable = false);
    __fastcall ~TImageStore();
    bool __fastcall Read(void * buffer, int nLength, unsigned *nBytesRead);
    bool __fastcall Write(void *buffer, int nLength, unsigned *nBytesWritten);
    bool __fastcall Seek(unsigned __int64 Offset, DWORD MoveMethod);
    void __fastcall LoadEmptyBlockMap(unsigned int BlockSize);
    bool __fastcall inline IsNextBlockEmpty(void) { if (EmptyBlockMap) return EmptyBlockMap[(unsigned int)(FPosition / FBlockSize)]; else return false; };

  protected:

  private:
    TImageStoreType    FType;
    AnsiString         FFileName;
    bool               bWriteAccess, bIsNTFSFAT32;
    HANDLE             hHandle;
    TNBDClient         *NBDConnection;
    TTargetGeometry    FGeometry;
    TCompressionFormat FCompressionFormat;
    unsigned __int64   FPosition;
    unsigned int       FBlockSize;
    bool               *EmptyBlockMap;

    bool __fastcall Open(void);
    bool __fastcall Close(void);
    void __fastcall CheckCompression(void);
    bool inline __fastcall GetCompressed(void) { return (FCompressionFormat != cfNone); };
    bool inline __fastcall GetBlockCompressed(void) { return (FCompressionFormat == cfGzipBlock || FCompressionFormat == cfBzip2Block); };

};  // class TImageStore
//---------------------------------------------------------------------------
#pragma pack(push,1)
struct TBootSector {
  unsigned char Jump[3];
  char          OEMID[8];
  WORD          BytesPerSector;
  BYTE          SectorsPerCluster;
  WORD          ReservedSectors;
  BYTE          NumberOfFATs;
  WORD          RootEntries;
  WORD          FAT_TotalSectors;
  BYTE          MediaDescriptor;
  WORD          SectorsPerFAT;
  WORD          SectorsPerTrack;
  WORD          Heads;
  DWORD         HiddenSectors;
  DWORD         FAT_BigTotalSectors;
  DWORD         BigSectorsPerFAT;
  LONGLONG      TotalSectors;
  LONGLONG      MFTCluster;
  LONGLONG      MFTMirrorCluster;
  DWORD         ClustersPerFileRecord;
  DWORD         ClustersPerIndexBlock;
  LONGLONG      VolumeSerial;
  DWORD         Checksum;
  unsigned char Bootstrap[426];
  WORD          EndOfSector;
};
#pragma pack(pop)
//---------------------------------------------------------------------------
#define NBD_MAGIC_CONNECT1 "NBDMAGIC"
#define NBD_MAGIC_CONNECT2 0x0000420281861253ui64
#define NBD_MAGIC_REQUEST 0x25609513
#define NBD_MAGIC_REPLY   0x67446698
//---------------------------------------------------------------------------
#ifndef ntohll
#define ntohll(x) (((__int64)(ntohl((int)((x << 32) >> 32))) << 32) | (unsigned int)ntohl(((int)(x >> 32))))
#endif
#ifndef htonll
#define htonll(x) ntohll(x)
#endif
//---------------------------------------------------------------------------
#endif
